package cz.petrdana.android.tachometer.activity;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

import android.content.Intent;
import android.database.Cursor;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentActivity;
import android.support.v4.app.FragmentTransaction;
import android.view.Menu;
import android.view.View;
import android.widget.TextView;
import android.widget.Toast;
import cz.petrdana.android.tachometer.R;
import cz.petrdana.android.tachometer.fragment.AddRecordFragment;
import cz.petrdana.android.tachometer.fragment.TachoListFragment;
import cz.petrdana.android.tachometer.model.Record;
import cz.petrdana.android.tachometer.model.TachoModel;

public class MainActivity extends FragmentActivity implements
        TachoListFragment.OnTachoItemClickedListener,
        AddRecordFragment.OnAddRecordListener {

    private static final int REQUEST_ADD_RECORD = 0;
    private static final int REQUEST_UPDATE_RECORD = 1;

    private boolean dualPane;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        // podle toho, jakou layout slozku system vybere, bude existovat right_column (w660dp) nebo ne
        dualPane = findViewById(R.id.right_column) != null;
        
        if (dualPane) {
            
        } else {
            // However, if we're being restored from a previous state,
            // then we don't need to do anything and should return or else
            // we could end up with overlapping fragments.
            if (savedInstanceState != null) {
                return;
            }

            Fragment firstFragment = new TachoListFragment();
            
            // In case this activity was started with special instructions from an Intent,
            // pass the Intent's extras to the fragment as arguments
            firstFragment.setArguments(getIntent().getExtras());
            
            // Add the fragment to the 'fragment_container' FrameLayout
            getSupportFragmentManager().beginTransaction()
                    .add(R.id.fragment_container, firstFragment).commit();
        }
    }


    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.main, menu);
        return true;
    }

    public void onAddRecordClicked(View v) {
        if (dualPane) {
            Fragment f = new AddRecordFragment();
            FragmentTransaction ft = getSupportFragmentManager().beginTransaction();
            ft.replace(R.id.right_column, f);
            ft.addToBackStack(null);
            ft.commit();
        } else {
            /*Intent i = new Intent(this, AddRecordActivity.class);
            startActivityForResult(i, REQUEST_ADD_RECORD);*/
            Fragment f = new AddRecordFragment();
            FragmentTransaction ft = getSupportFragmentManager().beginTransaction();
            ft.replace(R.id.fragment_container, f);
            ft.addToBackStack(null);
            ft.commit();
        }
    }

    @Override
    public void onAddRecord(Record record) {
        TachoModel tachoModel = new TachoModel(this);
        long id = 0;
        if (record.isNew) {
            id = tachoModel.insertRecord(record);
            if (id > -1) {
                //((TachoListFragment) getSupportFragmentManager().findFragmentById(R.id.tacho_list)).updateList();
                getSupportFragmentManager().popBackStack();

//                if (dualPane) {
//                    showNote(id);
//                }
            } else {
                Toast.makeText(this, R.string.record_not_added, Toast.LENGTH_LONG).show();
            }
        } else {
            boolean result = tachoModel.updateRecord(record);
            if (result) {
                getSupportFragmentManager().popBackStack();
            } else {
                Toast.makeText(this, R.string.record_not_added, Toast.LENGTH_LONG).show();
            }
        }
    }

    @Override
    public void onTachoItemClicked(long id) {

    }
    
    @Override
    public void onTachoItemUpdateClicked(long id) {
        if (dualPane){
            Fragment f = new AddRecordFragment();
            FragmentTransaction ft = getSupportFragmentManager().beginTransaction();
            ft.replace(R.id.right_column, f);
            ft.addToBackStack(null);
            ft.commit();
        } else {
            TachoModel tachoModel = new TachoModel(this);
            Cursor c = tachoModel.getRecord(id);
            c.moveToFirst();

            if (c.getCount() == 1) {
                int dateIndex = c.getColumnIndex(TachoModel.COLUMN_DATE);
                int dailyCounterIndex = c.getColumnIndex(TachoModel.COLUMN_DAILY_COUNTER);
                int mainCounterIndex = c.getColumnIndex(TachoModel.COLUMN_MAIN_COUNTER);
                int descriptionIndex = c.getColumnIndex(TachoModel.COLUMN_DESCRIPTION);
                
                Record record = new Record(
                    id,
                    c.getLong(dateIndex),
                    c.getFloat(dailyCounterIndex),
                    c.getInt(mainCounterIndex),
                    c.getString(descriptionIndex)
                );
                record.isNew = false;
                
                Fragment f = new AddRecordFragment();
                
                Bundle args = new Bundle();
                args.putParcelable("update_data", record);
                f.setArguments(args);
                
                FragmentTransaction ft = getSupportFragmentManager().beginTransaction();
                ft.replace(R.id.fragment_container, f);
                ft.addToBackStack(null);
                ft.commit();
            } else {
                Toast.makeText(this, R.string.record_not_found, Toast.LENGTH_LONG).show();
            }
            
            c.close();

            /*Intent i = new Intent(this, AddRecordActivity.class);
            i.putExtra(AddRecordActivity.EXTRA_RECORD, record);
            startActivityForResult(i, REQUEST_UPDATE_RECORD);*/
        }
    }

    @Override
    protected void onActivityResult (int requestCode, int resultCode, Intent data){
        if (requestCode == REQUEST_ADD_RECORD || requestCode == REQUEST_UPDATE_RECORD){
            if (resultCode != RESULT_OK) {
                return;
            }

            /*Record record = new Record(
                    data.getLongExtra(AddRecordActivity.EXTRA_DATE, 0),
                    data.getFloatExtra(AddRecordActivity.EXTRA_DAILY_COUNTER, 0.0f),
                    data.getIntExtra(AddRecordActivity.EXTRA_MAIN_COUNTER, 0),
                    data.getStringExtra(AddRecordActivity.EXTRA_DESCRIPTION)
            );
            record.isNew = (requestCode == REQUEST_ADD_RECORD);

            onAddRecord(record);*/
        }

        super.onActivityResult(requestCode, resultCode, data);
    }
}
