package cz.petrdana.android.tachometer.fragment;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;

import android.app.Activity;
import android.app.DatePickerDialog;
import android.os.Bundle;
import android.support.v4.app.DialogFragment;
import android.support.v4.app.Fragment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.DatePicker;
import android.widget.EditText;
import android.widget.TextView;
import cz.petrdana.android.tachometer.R;
import cz.petrdana.android.tachometer.model.Record;


public class AddRecordFragment extends Fragment implements DatePickerDialog.OnDateSetListener {
    
    final private static String STATE_DATE_MILLIS = "DATE_MILLIS";
    
    private OnAddRecordListener listener;
    private Calendar date;
    private boolean isNew = true;
    private long id;
    

    // ------------------------------------------------------------------------
    
    @Override
    public void onAttach(Activity activity) {
        super.onAttach(activity);
        try {
            listener = (OnAddRecordListener) activity;
        } catch (ClassCastException e) {
            throw new ClassCastException(activity.toString() + " must implement OnAddRecordListener");
        }
    }
    
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        //setRetainInstance(true);
        
        // vychozi datum bude aktualni, ale jen kdyz nebude nastaven v savedInstanceState
        this.date = Calendar.getInstance();
        
        if (savedInstanceState != null) {
            this.date.setTimeInMillis(savedInstanceState.getLong(STATE_DATE_MILLIS));
            //savedInstanceState.clear();
        }
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        
        View view = inflater.inflate(R.layout.add_record, null);

        TextView tvDate = (TextView)view.findViewById(R.id.tvDate);
        DateFormat df = SimpleDateFormat.getDateInstance();
        tvDate.setText(df.format(this.date.getTime()));
        
        Bundle args = getArguments();
        if (args != null) {
            Record record = args.getParcelable("update_data");
            if (record != null) {
                this.date.setTimeInMillis(record.dateInMillis);
                tvDate.setText(df.format(record.dateInMillis));
                
                this.isNew = false;
                this.id = record.id;
                
                
                ((EditText)view.findViewById(R.id.dailyCounter)).setText(String.valueOf(record.dailyCounter));
                ((EditText)view.findViewById(R.id.mainCounter)).setText(String.valueOf(record.mainCounter));
                ((EditText)view.findViewById(R.id.description)).setText(String.valueOf(record.description));
            }
        }
        
        Button pickDate = (Button)view.findViewById(R.id.pick_date);
        pickDate.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                onPickDateClicked();
            }
        });
        
        Button submit = (Button)view.findViewById(R.id.submit);
        submit.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                onSubmitClicked();
            }
        });

        return view;
    }
    
    // ------------------------------------------------------------------------
    
    @Override
    public void onSaveInstanceState(Bundle outState) {
        super.onSaveInstanceState(outState);
        outState.putLong(STATE_DATE_MILLIS, this.date.getTimeInMillis());
    }

    public void onSubmitClicked(){
        View root = getView();

        float dailyCounter = Float.valueOf(((EditText)root.findViewById(R.id.dailyCounter)).getText().toString());
        int mainCounter = Integer.valueOf(((EditText)root.findViewById(R.id.mainCounter)).getText().toString());
        String description = ((EditText)root.findViewById(R.id.description)).getText().toString();
        
        Record record = new Record(this.id, this.date.getTimeInMillis(), dailyCounter, mainCounter, description);
        record.isNew = this.isNew;
        listener.onAddRecord(record);
    }
    
    @Override
    public void onDateSet(DatePicker view, int year, int monthOfYear, int dayOfMonth) {
        this.date.set(year, monthOfYear, dayOfMonth);
        DateFormat df = SimpleDateFormat.getDateInstance();
        
        View root = getView();
        ((TextView) root.findViewById(R.id.tvDate)).setText(df.format(this.date.getTime()));
        
    }
    
    public void onPickDateClicked() {
        DialogFragment newFragment = DateDialogFragment.newInstance(this.date.getTimeInMillis());
        // http://stackoverflow.com/a/6672688 (http://stackoverflow.com/questions/6672066/fragment-inside-fragment)
        newFragment.show(getChildFragmentManager(), "datePicker");
    }

    // ------------------------------------------------------------------------
    
    public static interface OnAddRecordListener {
        public void onAddRecord(Record record);
    }
    
}
