package cz.petrdana.android.tachometer.fragment;

import android.app.Activity;
import android.content.Context;
import android.database.Cursor;
import android.os.Bundle;
import android.support.v4.app.ListFragment;
import android.util.Log;
import android.view.ContextMenu;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ListView;
import android.widget.Toast;
import cz.petrdana.android.tachometer.R;
import cz.petrdana.android.tachometer.adapter.TachoListAdapter;
import cz.petrdana.android.tachometer.model.TachoModel;

public class TachoListFragment extends ListFragment {

    private static final int MENU_EDIT_ID = 0;
    private static final int MENU_DELETE_ID = 1;
    
    private OnTachoItemClickedListener listener;
    private TachoModel model;
    private Cursor c;

    // ------------------------------------------------------------------------
    
    @Override
    public void onAttach(Activity activity) {
        super.onAttach(activity);
        Log.d("TEST", "onAttach");
        try {
            listener = (OnTachoItemClickedListener) activity;
        } catch (ClassCastException e) {
            throw new ClassCastException(activity.toString() + " must implement OnTachoItemClickedListener");
        }
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        Log.d("TEST", "onCreate");
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        super.onCreateView(inflater, container, savedInstanceState);
        Log.d("TEST", "onCreateView");
        View view = inflater.inflate(R.layout.tacho_list, container, false);
        return view;
    }

    @Override
    public void onActivityCreated(Bundle savedInstanceState) {
        super.onActivityCreated(savedInstanceState);
        registerForContextMenu(getListView());
        
        Log.d("TEST", "onActivityCreated");
        
        // toto musi byt v onStart, protoze kdyz vypnu app tlacikem Home a pak ji zase zapnu, tak se nezavola
        // onActivityCreated, ale az onStart
        
        /*Context ctx = getActivity();
        Log.d("TEST", "onActivityCreated: before new TachoModel");
        this.model = new TachoModel(ctx);
        Log.d("TEST", "onActivityCreated: after new TachoModel");
        
        updateList();*/
    }

    @Override
    public void onStart() {
        super.onStart();
        Log.d("TEST", "onStart");
        
        // toto musi byt v onStart, protoze kdyz vypnu app tlacikem Home a pak ji zase zapnu, tak se nezavola
        // onActivityCreated, ale az onStart
        Context ctx = getActivity();
        Log.d("TEST", "onStart: before new TachoModel");
        this.model = new TachoModel(ctx);
        Log.d("TEST", "onStart: after new TachoModel");
        
        updateList();
    }
    
    @Override
    public void onResume() {
        super.onResume();
        Log.d("TEST", "onResume");
    }
    
    @Override
    public void onPause() {
        Log.d("TEST", "onPause: before test isClosed");
        if (!this.c.isClosed()) {
            Log.d("TEST", "onPause: before cursor close");
            this.c.close();
            Log.d("TEST", "onPause: after cursor close");
        }
        Log.d("TEST", "onPause: after test isClosed");
        
        Log.d("TEST", "onPause: before model close");
        this.model.close();
        Log.d("TEST", "onPause: after model close");
        //getActivity().stopManagingCursor(this.c);
        
        super.onPause();
    }
    
    @Override
    // toto neni vylozene metoda v ramci zivotniho cyklu, ale v jedne knize to takto meli,
    // nejspis se vola mezi onPause a onStop, ikdyz stoprocentni to asi nebude
    public void onSaveInstanceState(Bundle savedInstanceState) {
        Log.d("TEST", "onSaveInstanceState");
        super.onSaveInstanceState(savedInstanceState);
    }
    
    @Override
    public void onStop() {
        Log.d("TEST", "onStop");
        super.onStop();
    }
    
    @Override
    public void onDestroyView() {
        Log.d("TEST", "onDestroyView");
        super.onDestroyView();
    }
    
    @Override
    public void onDestroy() {
        //getActivity().stopManagingCursor(this.c);
        
        Log.d("TEST", "onDestroy");
        super.onDestroy();
    }
    
    @Override
    public void onDetach() {
        Log.d("TEST", "onDetach");
        super.onDetach();
    }
    
    // ------------------------------------------------------------------------
    
    @Override
    public void onListItemClick(ListView l, View v, int position, long id) {
        super.onListItemClick(l, v, position, id);
        listener.onTachoItemClicked(id);
    }

    @Override
    public void onCreateContextMenu(ContextMenu menu, View v, ContextMenu.ContextMenuInfo menuInfo) {
        super.onCreateContextMenu(menu, v, menuInfo);
        menu.add(Menu.NONE, MENU_EDIT_ID, Menu.NONE, R.string.edit);
        menu.add(Menu.NONE, MENU_DELETE_ID, Menu.NONE, R.string.delete);
    }

    @Override
    public boolean onContextItemSelected(MenuItem item) {
        AdapterView.AdapterContextMenuInfo info = (AdapterView.AdapterContextMenuInfo) item.getMenuInfo();
        if (info == null) {
            return super.onContextItemSelected(item);
        }
        switch (item.getItemId()) {
            case MENU_EDIT_ID:
                editItem(info.id);
                return true;
            case MENU_DELETE_ID:
                deleteItem(info.id);
                return true;
            default:
                return super.onContextItemSelected(item);
        }
    }

    // ------------------------------------------------------------------------
    
    public void updateList() {
        Context ctx = getActivity();

        /*String[] from = { TachoModel.COLUMN_DATE };
        int[] to = { android.R.id.text1 };*/

        Log.d("TEST", "updateList: before getRecords");
        this.c = this.model.getRecords();
        Log.d("TEST", "updateList: after getRecords");
        //getActivity().startManagingCursor(this.c);
        
        TachoListAdapter adapter = new TachoListAdapter(ctx, this.c, 0);
        
        /*ListAdapter adapter = new SimpleCursorAdapter(ctx,
                android.R.layout.simple_list_item_1, notes.getRecords(), from,
                to, 0);*/

        setListAdapter(adapter);

        //c.close();
        //model.close();
    }

    private void editItem(long id) {
        //TODO: editace polozky
        //Context ctx = getActivity();
        //Toast.makeText(ctx, "edit item", Toast.LENGTH_SHORT).show();
        
        listener.onTachoItemUpdateClicked(id);
    }
    
    private void deleteItem(long id) {
        Context ctx = getActivity();
        if (this.model.deleteRecord(id)) {
            Toast.makeText(ctx, R.string.deleted, Toast.LENGTH_SHORT).show();
            updateList();
        } else {
            Toast.makeText(ctx, R.string.not_deleted, Toast.LENGTH_SHORT).show();
        }
    }
    
    // ------------------------------------------------------------------------
    
    public static interface OnTachoItemClickedListener {
        public void onTachoItemClicked(long id);
        public void onTachoItemUpdateClicked(long id);
    }

}