package cz.petrdana.android.tachometer.model;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.os.Parcel;
import android.os.Parcelable;

public class TachoModel {

    protected static final String DATABASE_NAME = "tachometer";
    protected static final int DATABASE_VERSION = 2;

    protected static final String TB_NAME = "records";

    // Speciální hodnota "_id", pro jednodušší použití SimpleCursorAdapteru
    public static final String COLUMN_ID = "_id";
    public static final String COLUMN_DATE = "date";
    public static final String COLUMN_DAILY_COUNTER = "daily_counter";
    public static final String COLUMN_MAIN_COUNTER = "main_counter";
    public static final String COLUMN_DESCRIPTION = "description";

    public static final String[] columns =
            { COLUMN_ID, COLUMN_DATE, COLUMN_DAILY_COUNTER, COLUMN_MAIN_COUNTER, COLUMN_DESCRIPTION };
    protected static final String ORDER_BY = COLUMN_DATE + " DESC, " + COLUMN_ID + " DESC";

    private SQLiteOpenHelper openHelper;

    static class DatabaseHelper extends SQLiteOpenHelper {

        DatabaseHelper(Context context) {
            super(context, DATABASE_NAME, null, DATABASE_VERSION);
        }

        @Override
        public void onCreate(SQLiteDatabase db) {
            db.execSQL("CREATE TABLE " + TB_NAME + " ("
                    + COLUMN_ID + " INTEGER PRIMARY KEY,"
                    + COLUMN_DATE + " INTEGER NOT NULL,"
                    + COLUMN_DAILY_COUNTER + " REAL NOT NULL,"
                    + COLUMN_MAIN_COUNTER + " INTEGER NOT NULL,"
                    + COLUMN_DESCRIPTION + " TEXT NOT NULL"
                    + ");");
        }

        /*
         * Ve skutečnosti je potřeba, abychom uživatelům nemazali data, vytvořit
         * pro každou změnu struktury databáze nějaký upgradovací nedestruktivní
         * SQL příkaz.
         */
        @Override
        public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
            if (oldVersion == 1) {
                // SQLite neumi modifikovat sloupce, musi se vytvorit nova tabulka a data do ni zkopirovat, coz pri
                // zmene typu sloupce muze byt opruz.
                // V tomto pripade je to jeste na zacatku vyvoje, takze tabulku proste smazeme i s daty a nechame
                // vytrorit znova zavolanim upraveneho onCreate.
                db.execSQL("DROP TABLE IF EXISTS " + TB_NAME);
                onCreate(db);
            }
        }
    }

    public TachoModel(Context ctx) {
        openHelper = new DatabaseHelper(ctx);
    }
    
    public Cursor getRecords() {
        SQLiteDatabase db = openHelper.getReadableDatabase();
        return db.query(TB_NAME, columns, null, null, null, null, ORDER_BY);
    }

    public Cursor getRecord(long id) {
        SQLiteDatabase db = openHelper.getReadableDatabase();
        String[] selectionArgs = { String.valueOf(id) };
        return db.query(TB_NAME, columns, COLUMN_ID + "= ?", selectionArgs,
                null, null, null);
    }

    public long insertRecord(Record record) {
        SQLiteDatabase db = openHelper.getWritableDatabase();

        ContentValues values = new ContentValues();
        values.put(COLUMN_DATE, record.dateInMillis);
        values.put(COLUMN_DAILY_COUNTER, record.dailyCounter);
        values.put(COLUMN_MAIN_COUNTER, record.mainCounter);
        values.put(COLUMN_DESCRIPTION, record.description);

        long id = -1;
        if (db != null) {
            id = db.insert(TB_NAME, null, values);
            db.close();
        }
        return id;
    }
    
    public boolean updateRecord(Record record) {
        SQLiteDatabase db = openHelper.getWritableDatabase();
        String[] selectionArgs = { String.valueOf(record.id) };
        
        ContentValues values = new ContentValues();
        values.put(COLUMN_DATE, record.dateInMillis);
        values.put(COLUMN_DAILY_COUNTER, record.dailyCounter);
        values.put(COLUMN_MAIN_COUNTER, record.mainCounter);
        values.put(COLUMN_DESCRIPTION, record.description);
        
        int updateCount = db.update(TB_NAME, values, COLUMN_ID + "= ?", selectionArgs);
        db.close();
        return updateCount > 0;
    }
    
    public boolean deleteRecord(long id) {
        SQLiteDatabase db = openHelper.getWritableDatabase();
        String[] selectionArgs = { String.valueOf(id) };
        int deleteCount = db.delete(TB_NAME, COLUMN_ID + "= ?", selectionArgs);
        db.close();
        return deleteCount > 0;
    }

    public void close() {
        openHelper.close();
    }

}
